<?php

namespace Drupal\vite_react_generator\Drush\Generators;

use Drupal\vite_react_generator\Drush\Generators\Validator\RequiredComponentName;
use DrupalCodeGenerator\Asset\Asset;
use DrupalCodeGenerator\Asset\AssetCollection;
use DrupalCodeGenerator\Attribute\Generator;
use DrupalCodeGenerator\Command\BaseGenerator;
use DrupalCodeGenerator\GeneratorType;
use DrupalCodeGenerator\Utils;
use DrupalCodeGenerator\Validator\Required;
use Symfony\Component\Yaml\Yaml;

/**
 * Generates a Vite + React SDC module component.
 */
#[Generator(
  name: 'single-directory-component-vite-react',
  description: 'Generates a Vite + React SDC module component.',
  aliases: ['sdc-vite-react'],
  templatePath: __DIR__ . '/../../../templates',
  type: GeneratorType::MODULE_COMPONENT,
)]
final class ReactComponentGenerator extends BaseGenerator {
  private const TRANSPILER_BABEL = 'babel';
  private const TRANSPILER_SWC = 'swc';
  private const TRANSPILER_CHOICES = [
    self::TRANSPILER_SWC => 'Speedy Web Compiler',
    self::TRANSPILER_BABEL => 'Babel',
  ];

  /**
   * {@inheritdoc}
   */
  public function generate(array &$vars, AssetCollection $assets): void {
    $this->askQuestions($vars);

    $fullModulePath = $this->getHelper('module_info')->getDestination($vars['machine_name'], FALSE);
    $vars['module_path'] = str_replace(\DRUPAL_ROOT . DIRECTORY_SEPARATOR, '', $fullModulePath);

    $packageJsonFilePath = sprintf('%s/%s/package.json', $this->getTemplatePath(), $vars['transpiler']);
    $vars['package_json'] = json_decode(file_get_contents($packageJsonFilePath), TRUE);

    $this->generateAssets($vars, $assets);
  }

  /**
   * {@inheritdoc}
   */
  protected function render(AssetCollection $assets): void {
    parent::render($assets);

    $infoAssets = $assets->getFiltered(fn (Asset $asset) => str_ends_with(
      $asset->getPath(), $asset->getVars()['machine_name'] . '.info.yml'
    ));
    if (count($infoAssets) !== 1) {
      return;
    }

    /** @var \DrupalCodeGenerator\Asset\File $asset */
    $asset = $infoAssets[0];
    $data = Yaml::parse($asset->getContent());

    $vars = $asset->getVars();
    $infoFilePath = sprintf('%s/%s.info.yml', $vars['module_path'], $vars['machine_name']);
    if (file_exists($infoFilePath)) {
      $data = array_merge($data, Yaml::parseFile($infoFilePath));
    }

    $data['core_version_requirement'] = '^10 || ^11';
    $data['interface translation project'] ??= $vars['machine_name'];
    $data['interface translation server pattern'] ??= sprintf('%s/translations/%s.%%language.po', $vars['module_path'], $vars['machine_name']);

    $asset->content(Yaml::dump($data));
    $asset->replaceIfExists();
  }

  /**
   * Function to get all file paths relative to the base directory.
   *
   * @param string $baseDir
   *   The base directory to start the search.
   *
   * @return array
   *   An array of relative file paths.
   */
  public function getAllRelativeFilePaths(string $baseDir): array {
    $relativeFilePaths = [];

    $directoryIterator = new \RecursiveDirectoryIterator($baseDir);
    $iterator = new \RecursiveIteratorIterator($directoryIterator);

    foreach ($iterator as $file) {
      if ($file->isFile()) {
        $relativeFilePaths[] = \str_replace($baseDir . '/', '', $file->getPathname());
      }
    }

    return $relativeFilePaths;
  }

  /**
   * Collects user answers.
   *
   * @param array &$vars
   *   The variables array with the user answers.
   */
  private function askQuestions(array &$vars): void {
    $ir = $this->createInterviewer($vars);

    $vars['machine_name'] = $ir->askMachineName();
    $vars['name'] = $ir->askName();

    $vars['component_name'] = $ir->ask(
      'Component name',
      \ucwords(Utils::machine2human($vars['machine_name'])),
      new Required(),
    );
    $vars['component_machine_name'] = $ir->ask(
      'Component machine name',
      Utils::human2machine($vars['component_name']),
      new RequiredComponentName(),
    );

    $vars['component_description'] = $ir->ask('Component description (optional)');

    $vars['transpiler'] = $ir->choice('Transpiler', self::TRANSPILER_CHOICES, self::TRANSPILER_CHOICES[self::TRANSPILER_SWC]);
    $vars['install_testing_frameworks'] = $ir->confirm('Install testing frameworks?', default: FALSE);
  }

  /**
   * Create the assets that the framework will write to disk later on.
   *
   * @param array $vars
   *   The variables array with the user answers.
   * @param \DrupalCodeGenerator\Asset\AssetCollection $assets
   *   The generate assets.
   */
  private function generateAssets(array $vars, AssetCollection $assets): void {
    $templatePath = $this->getTemplatePath();
    if (!isset($templatePath)) {
      throw new \RuntimeException('Template path not found');
    }

    $baseDir = sprintf("%s/_base", $templatePath);
    foreach ($this->getAllRelativeFilePaths($baseDir) as $template) {
      if (str_starts_with($template, 'node_modules/')) {
        continue;
      }

      $relativeTemplatePath = sprintf("_base/%s", $template);
      if (str_ends_with($template, '.twig')) {
        $output = substr($template, 0, -5);
        $output = preg_replace('/\[(\w+)]/', '{\\1}', $output);
        $assets->addFile($output, $relativeTemplatePath);
      }
      else {
        $file = $assets->addFile($template);
        $content = file_get_contents(sprintf('%s/%s', $templatePath, $relativeTemplatePath));
        $file->content($content);
      }
    }
  }

}
